%butterfly
% Plots the Lorenz Attractor

function butterfly

%Lorenz parameters
s = 10;
r = 28; 
b = 8/3;

N = 5000; %Number of trajectory points to plot
M = 50; %Number of trajectories
dt = 0.005; %Time interval
fsize = 16; %Fontsize

%Define interval for (random) initial x,y,z trajectory starting points.
x0min = -1; x0max = 1; y0min = -1; y0max = 1; z0min = -1; z0max = 1;

%

%Initialize arrays
t = 0 : dt : (N-1)*dt; t = t.'; t = repmat( t, [1,M] );
x = x0min + rand(1,M)*( x0max - x0min ); x = repmat( x, [N,1] );
y = y0min + rand(1,M)*( y0max - y0min ); y = repmat( y, [N,1] );
z = z0min + rand(1,M)*( z0max - z0min ); z = repmat( z, [N,1] );

%Walk through the arrays, and use the Runge-Kutta method to determine what
%happens next
for n=1:N-1
    [xdot1,ydot1,zdot1] = f( x(n,:),y(n,:),z(n,:),s,r,b );
    [xdot2,ydot2,zdot2] = f( x(n,:) + 0.5*dt*xdot1,...
        y(n,:) + 0.5*dt*ydot1,...
        z(n,:) + 0.5*dt*zdot1,...
        s,r,b );
    [xdot3,ydot3,zdot3] = f( x(n,:) + 0.5*dt*xdot2,...
        y(n,:) + 0.5*dt*ydot2,...
        z(n,:) + 0.5*dt*zdot2,...
        s,r,b );
    [xdot4,ydot4,zdot4] = f( x(n,:) + dt*xdot3,...
        y(n,:) + dt*ydot3,...
        z(n,:) + dt*zdot3,...
        s,r,b );
    x(n+1,:) = x(n,:) + (dt/6)* (xdot1 + 2*xdot2 + 2*xdot3 + xdot4 );
    y(n+1,:) = y(n,:) + (dt/6)* (ydot1 + 2*ydot2 + 2*ydot3 + ydot4 );
    z(n+1,:) = z(n,:) + (dt/6)* (zdot1 + 2*zdot2 + 2*zdot3 + zdot4 );   
end

%Plot trajectories
plot3(x,y,z); axis equal; axis vis3d; view([50,32]); grid on; box on;
set(gca,'fontsize',fsize);
xlabel('x'); ylabel('y'); zlabel('z'); title('Lorenz attractor');
print(gcf,'butterfly.png','-dpng','-r300'); close(gcf);

%%

%Lorenz equations
function [xdot,ydot,zdot] = f(x,y,z,s,r,b)
xdot = s*( y - x );
ydot = x.*(r - z) - y;
zdot = x.*y -b*z;

%End of code
